
TILESIZE = 32

collisionMap = {} -- we will store our collision data here

local map = loadfile("tilemap.lua")() -- load tilemap

tileMap = map.layers[2].data -- tunnels is the 2nd layer

MAP_HEIGHT = map.width
MAP_WIDTH = map.height

atlas = Atlas.new("tiles.png", false) -- a texture atlas containing our tile images
local region = atlas:newRegion(128) -- get the dirt texture
local brush = Brush.new(region) -- we will show this on stage
local canvas = RenderTarget.new(MAP_WIDTH * TILESIZE, MAP_HEIGHT * TILESIZE) -- we will draw here
local display = Bitmap.new(canvas) -- we will draw to the canvas with this

local function resetCanvas() -- flood fill the canvas with dirt texture
  for r = 0, MAP_HEIGHT - 1 do
    for c = 0, MAP_WIDTH - 1 do
      brush:setPosition(c * TILESIZE, r * TILESIZE)
      canvas:draw(brush)
    end
  end
end

local function drawTunnels() -- draw the tunnels over the dirt
  for r = 1, MAP_HEIGHT do
    local y = (r - 1) * MAP_WIDTH
    for c = 1, MAP_WIDTH do
      local tile = tileMap[y + c]
      if tile > 0 then
        brush:setPosition((c - 1) * TILESIZE, (r - 1) * TILESIZE)
        brush:setImage(atlas:getRegion(tile))
		canvas:draw(brush)
      end
    end
  end
end

local function clearCollisionMap() -- recreate the grid of cells that make up the collision map
  local b = {}
  for r = 1, MAP_HEIGHT do
    local row = {}
    for c = 1, MAP_WIDTH do
      row[#row + 1] = {t = 0, l = false, r = false, u = false, d = false} -- create row
    end
    b[#b + 1] = row -- store row
  end
  collisionMap = b
end

local function remakeCollisionMap() -- create collision matrix
  local doors = {
  -- l      r      u      d
    {false, false, false, false},
    {false, false, true,  false},
    {true,  false, false, false},
    {true,  false, true,  false},
    {false, false, false, true}, 
    {false, false, true,  true}, 
    {true,  false, false, true}, 
    {true,  false, true,  true}, 
    {false, true,  false, false},
    {false, true,  true,  false},
    {true,  true,  false, false},
    {true,  true,  true,  false},
    {false, true,  false, true}, 
    {false, true,  true,  true}, 
    {true,  true,  false, true}, 
    {true,  true,  true,  true}, 
  }
  
  for r = 1, MAP_HEIGHT do
    local y = (r - 1) * MAP_WIDTH
    for c = 1, MAP_WIDTH do
      local tile = tileMap[y + c]   --[r][c]
      if tile > 0 then
        local door = doors[tile]
        local cell = collisionMap[r][c]
        cell.t = tile -- set tile
        cell.l = door[1] -- set travel directions from this cell
        cell.r = door[2]
        cell.u = door[3]
        cell.d = door[4]
      end
    end
  end
end

clearCollisionMap()
remakeCollisionMap()
resetCanvas()
drawTunnels()

agent = Agent.new(atlas:newRegion(126)) -- create a simple agent that we can move about the map
agent:setLocation(8, 7) -- set location in grid
local objects = map.layers[3].objects
agent:setLocation(objects[1].x, objects[1].y)

local L = Button.new(atlas:newRegion(122), { -- create some buttons we can use to move the actor
  x = 32,
  y = 432,
  onPressed = function(e)
    agent:changeDirection(1) -- tell agent to move left
  end,
})

local R = Button.new(atlas:newRegion(123), {
  x = 128,
  y = 432,
  onPressed = function(e)
    agent:changeDirection(2) -- tell agent to move right
  end,
})

local U = Button.new(atlas:newRegion(124), {
  x = 80,
  y = 384,
  onPressed = function(e)
    agent:changeDirection(3) -- tell agent to move up
  end,
})

local D = Button.new(atlas:newRegion(125), {
  x = 80,
  y = 480,
  onPressed = function(e)
    agent:changeDirection(4) -- tell agent to move down
  end,
})

stage:addChild(display)
stage:addChild(agent)
stage:addChild(L)
stage:addChild(R)
stage:addChild(U)
stage:addChild(D)

local function onEnterFrame(e) -- update every frame
  agent:update()
end

stage:addEventListener(Event.ENTER_FRAME, onEnterFrame)
